@@ -42,8 +42,8 @@ int line_number;
 
 typedef struct
 {
-	char	*keyword;
-	int	type;
+	const char	*keyword;
+	int		type;
 	union {
 		void	*v;	/* join of other values */
 		int	*i;
@@ -51,6 +51,9 @@ typedef struct
 		char	**s;
 		color_t	*c;
 		double	*d;
+		stringlist_t *sl;
+		condstring_t *cs;
+		colorlist_t *cl;
 		INTTYPE	val;
 	} confref;
 } keyword_t;
@@ -67,18 +70,6 @@ static YYSTYPE yylval;
 static int nstacked_opts;
 static char **stacked_opts;
 
-enum {
-	TYPE_dummy = 256,
-	TYPE_KEYWORD,
-	TYPE_NUMBER,
-	TYPE_VALUE,
-	TYPE_BOOLEAN,
-	TYPE_COLOR,
-	TYPE_FONT,
-	TYPE_STRING,
-	TYPE_DOUBLE
-};
-
 static keyword_t keywords[] = {
 	{ "branch_bgcolor",	TYPE_COLOR,	{ &conf.branch_bgcolor } },
 	{ "branch_bspace",	TYPE_NUMBER,	{ &conf.branch_bspace } },
@@ -133,15 +124,16 @@ static keyword_t keywords[] = {
 	{ "rev_rspace",		TYPE_NUMBER,	{ &conf.rev_rspace } },
 	{ "rev_tspace",		TYPE_NUMBER,	{ &conf.rev_tspace } },
 	{ "rev_bspace",		TYPE_NUMBER,	{ &conf.rev_bspace } },
-	{ "rev_text",		TYPE_STRING,	{ &conf.rev_text } },
+	{ "rev_text",		TYPE_CSTRING,	{ &conf.rev_text } },
+	{ "rev_idtext",		TYPE_CSTRING,	{ &conf.rev_idtext } },
 	{ "rev_text_color",	TYPE_COLOR,	{ &conf.rev_text_color } },
 	{ "rev_text_font",	TYPE_FONT,	{ &conf.rev_text_font.gdfont } },
 	{ "rev_text_ttfont",	TYPE_STRING,	{ &conf.rev_text_font.ttfont } },
 	{ "rev_text_ttsize",	TYPE_DOUBLE,	{ &conf.rev_text_font.ttsize } },
 	{ "rev_maxtags",	TYPE_NUMBER,	{ &conf.rev_maxtags } },
-	{ "merge_color",	TYPE_COLOR,	{ &conf.merge_color } },
-	{ "merge_from",		TYPE_STRING,	{ &conf.merge_from } },
-	{ "merge_to",		TYPE_STRING,	{ &conf.merge_to } },
+	{ "merge_color",	TYPE_COLORLIST,	{ &conf.merge_color } },
+	{ "merge_from",		TYPE_STRINGLIST,	{ &conf.merge_from } },
+	{ "merge_to",		TYPE_STRINGLIST,	{ &conf.merge_to } },
 	{ "merge_findall",	TYPE_BOOLEAN,	{ &conf.merge_findall } },
 	{ "merge_front",	TYPE_BOOLEAN,	{ &conf.merge_front } },
 	{ "merge_nocase",	TYPE_BOOLEAN,	{ &conf.merge_nocase } },
@@ -289,10 +281,74 @@ static void debug_pfont(const char *n, gdFontPtr f)
 	fprintf(DEBUGSTREAM, "%s\n", s);
 }
 
+static char *debug_op[] = { "=~", "=*", "!~", "!*", "==", "!_", ">=", ">", "<=", "<" };
+
+static void debug_pcolornode(node_t *n)
+{
+	if(!n)
+		return;
+	if(n->op == TYPE_STRING)
+		fprintf(DEBUGSTREAM, "%s ", n->value.str);
+	else if(n->op == TYPE_COLOR)
+		fprintf(DEBUGSTREAM, "#%02x%02x%02x ", n->value.clr.r, n->value.clr.g, n->value.clr.b);
+	else
+	{
+		char *key;
+		fprintf(DEBUGSTREAM, "[ ");
+		switch(n->key)
+		{
+		case KEY_STATE:		key = "state"; break;
+		case KEY_AUTHOR:	key = "author"; break;
+		case KEY_TAG:		key = "tag"; break;
+		case KEY_DATE:		key = "date"; break;
+		case KEY_REV:		key = "rev"; break;
+		case KEY_UNKNOWN:	key = "unknown"; break;
+		default:		key = "<undefined>"; break;
+		}
+		fprintf(DEBUGSTREAM, "%s ", key);
+		if(n->op > OP_FIRST && n->op < OP_LAST)
+			fprintf(DEBUGSTREAM, "%s ", debug_op[n->op - OP_FIRST - 1]);
+		else
+			fprintf(DEBUGSTREAM, "op(-?-) ");
+		fprintf(DEBUGSTREAM, "%s ", n->content);
+		debug_pcolornode(n->tcase);
+		debug_pcolornode(n->fcase);
+		fprintf(DEBUGSTREAM, "]");
+	}
+}
+
 static void debug_pcolor(const char *n, color_t *c)
 {
 	debug_pname(n);
-	fprintf(DEBUGSTREAM, "#%02x%02x%02x\n", c->r, c->g, c->b);
+	if(c->node)
+	{
+		debug_pcolornode(c->node);
+		fprintf(DEBUGSTREAM, "\n");
+	}
+	else
+		fprintf(DEBUGSTREAM, "#%02x%02x%02x\n", c->r, c->g, c->b);
+}
+
+static void debug_pcolorlist(const char *n, colorlist_t *cl)
+{
+	int i;
+	char buf[128];
+	for(i = 0; i < cl->n; i++)
+	{
+		sprintf(buf, "%s{%d}", n, i);
+		debug_pcolor(buf, &cl->clrs[i]);
+	}
+}
+
+static void debug_pstringlist(const char *n, stringlist_t *sl)
+{
+	int i;
+	char buf[128];
+	for(i = 0; i < sl->n; i++)
+	{
+		sprintf(buf, "%s{%d}", n, i);
+		debug_pstring(buf, sl->strs[i]);
+	}
 }
 
 void dump_config(void)
@@ -311,11 +367,36 @@ void dump_config(void)
 	debug_pbool("anti_alias", conf.anti_alias);
 	debug_pbool("use_ttf", conf.use_ttf);
 	debug_pint("thick_lines", conf.thick_lines);
+	debug_pint("html_level", conf.html_level);
+	debug_pbool("parse_logs", conf.parse_logs);
+	debug_pbool("transparent_bg", conf.transparent_bg);
+
+	debug_pint("arrow_length", conf.arrow_length);
+	debug_pint("arrow_width", conf.arrow_width);
+
+	debug_pbool("merge_arrows", conf.merge_arrows);
+	debug_pcolor("merge_cvsnt_color", &conf.merge_cvsnt_color);
+	debug_pbool("merge_cvsnt", conf.merge_cvsnt);
+	debug_pbool("merge_findall", conf.merge_findall);
+	debug_pcolorlist("merge_color", &conf.merge_color);
+	debug_pstringlist("merge_from", &conf.merge_from);
+	debug_pstringlist("merge_to", &conf.merge_to);
+	debug_pbool("merge_front", conf.merge_front);
+	debug_pbool("merge_nocase", conf.merge_nocase);
+
+	debug_pcolor("msg_color", &conf.msg_color);
+	debug_pfont("msg_font", conf.msg_font.gdfont);
+	debug_pstring("msg_ttfont", conf.msg_font.ttfont);
+	debug_pdouble("msg_ttsize", conf.msg_font.ttsize);
 
 	debug_pfont("tag_font", conf.tag_font.gdfont);
 	debug_pstring("tag_ttfont", conf.tag_font.ttfont);
-	debug_psouble("tag_ttsize", conf.tag_font.ttsize);
+	debug_pdouble("tag_ttsize", conf.tag_font.ttsize);
 	debug_pcolor("tag_color", &conf.tag_color);
+	debug_pbool("tag_ignore_merge", conf.tag_ignore_merge);
+	debug_pstring("tag_ignore", conf.tag_ignore);
+	debug_pbool("tag_negate", conf.tag_negate);
+	debug_pbool("tag_nocase", conf.tag_nocase);
 
 	debug_pfont("rev_font", conf.rev_font.gdfont);
 	debug_pstring("rev_ttfont", conf.rev_font.ttfont);
@@ -325,15 +406,17 @@ void dump_config(void)
 	debug_pint("rev_separator", conf.rev_separator);
 	debug_pint("rev_minline", conf.rev_minline);
 	debug_pint("rev_maxline", conf.rev_maxline);
+	debug_pint("rev_maxtags", conf.rev_maxtags);
 	debug_pint("rev_lspace", conf.rev_lspace);
 	debug_pint("rev_rspace", conf.rev_rspace);
 	debug_pint("rev_tspace", conf.rev_tspace);
 	debug_pint("rev_bspace", conf.rev_bspace);
-	debug_pstring("rev_text", conf.rev_text);
+	debug_pcstring("rev_text", conf.rev_text);
 	debug_pcolor("rev_text_color", &conf.rev_text_color);
 	debug_pfont("rev_text_font", conf.rev_text_font.gdfont);
 	debug_pstring("rev_text_ttfont", conf.rev_text_font.ttfont);
 	debug_pdouble("rev_text_ttsize", conf.rev_text_font.ttsize);
+	debug_pbool("rev_hidenumber", conf.rev_hidenumber);
 
 	debug_pfont("branch_font", conf.branch_font.gdfont);
 	debug_pstring("branch_ttfont", conf.branch_font.ttfont);
@@ -351,6 +434,10 @@ void dump_config(void)
 	debug_pint("branch_connect", conf.branch_connect);
 	debug_pint("branch_margin", conf.branch_margin);
 	debug_pint("branch_dupbox", conf.branch_dupbox);
+	debug_pbool("branch_fold", conf.branch_fold);
+	debug_pbool("branch_foldall", conf.branch_foldall);
+	debug_pbool("branch_resort", conf.branch_resort);
+	debug_pstring("branch_subtree", conf.branch_subtree);
 
 	debug_pstring("title", conf.title);
 	debug_pint("title_x", conf.title_x);
@@ -368,6 +455,8 @@ void dump_config(void)
 
 	debug_pint("image_type", conf.image_type);
 	debug_pint("image_quality", conf.image_quality);
+	debug_pint("image_compress", conf.image_compress);
+	debug_pbool("image_interlace", conf.image_interlace);
 
 	debug_pstring("map_name", conf.map_name);
 	debug_pstring("map_branch_href", conf.map_branch_href);
@@ -376,6 +465,8 @@ void dump_config(void)
 	debug_pstring("map_rev_alt", conf.map_rev_alt);
 	debug_pstring("map_diff_href", conf.map_diff_href);
 	debug_pstring("map_diff_alt", conf.map_diff_alt);
+	debug_pstring("map_merge_href", conf.map_merge_href);
+	debug_pstring("map_merge_alt", conf.map_merge_alt);
 
 	debug_pstring("expand[0]", conf.expand[0]);
 	debug_pstring("expand[1]", conf.expand[1]);
@@ -511,9 +602,51 @@ static int config_lex(void)
 
 		switch(ch)
 		{
-		case EOF:
 		case '=':
+			ch = get_input();
+			switch(ch)
+			{
+			case '~': return OP_CONTAINED;
+			case '*': return OP_CONTAINEDI;
+			case '=': return OP_EQ;
+			default:
+				unget_input(ch);
+				return '=';
+			}
+			break;
+
+		case '!':
+			ch = get_input();
+			switch(ch)
+			{
+			case '~': return OP_NCONTAINED;
+			case '*': return OP_NCONTAINEDI;
+			case '=': return OP_NE;
+			default:
+				stack_msg(MSG_ERR, "config: %d: Invalid operator", line_number);
+				unget_input(ch);
+				return '!';
+			}
+			break;
+
+		case '>':
+			ch = get_input();
+			if(ch == '=')
+				return OP_GE;
+			unget_input(ch);
+			return OP_GT;
+
+		case '<':
+			ch = get_input();
+			if(ch == '=')
+				return OP_LE;
+			unget_input(ch);
+			return OP_LT;
+
+		case EOF:
 		case ';':
+		case '[':
+		case ']':
 			return ch;
 
 		case '#':	/* Comment */
@@ -616,7 +749,7 @@ static int config_lex(void)
 					yylval.kw = NULL;
 					return TYPE_KEYWORD;
 				}
-				xfree(skw.keyword);
+				xfree((void *)skw.keyword);
 				if(kw->type == TYPE_VALUE)
 				{
 					yylval.i = (int)kw->confref.val;
@@ -672,6 +805,7 @@ static int config_lex(void)
 static void set_color(color_t *c, char *s)
 {
 	char *cptr;
+	c->node = NULL;
 	if(*s != '#' || strlen(s) != 7)
 	{
 colorerror:
@@ -717,16 +851,139 @@ static gdFontPtr get_font(int id)
  *		;
  *
  *	line	: <keyword> '=' <value> ';'
+ *		| <keyword> '=' expr ';'
  *		| ';'
+ *
+ *	expr	: '[' <kw> <op> <value> expr expr ']'
+ *		| <value>
  *		;
  **************************************************************************
  */
+static int skip_to_semicolon(int state)
+{
+	int token;
+	while(1)
+	{
+		token = config_lex();
+		if(token == ';')
+			return 0;
+		else if(token == EOF)
+			return state;
+	}
+}
+
+static node_t *new_node(char *val)
+{
+	node_t *n = xmalloc(sizeof(*n));
+	if(!strcmp(val, "state"))
+		n->key = KEY_STATE;
+	else if(!strcmp(val, "author"))
+		n->key = KEY_AUTHOR;
+	else if(!strcmp(val, "tag"))
+		n->key = KEY_TAG;
+	else if(!strcmp(val, "date"))
+		n->key = KEY_DATE;
+	else if(!strcmp(val, "rev"))
+		n->key = KEY_REV;
+	else
+	{
+		n->key = KEY_UNKNOWN;
+		stack_msg(MSG_ERR, "config: %d: Unknown key '%s'", line_number, val);
+	}
+	return n;
+}
+
+typedef struct __statestack_t
+{
+	int	state;
+	node_t	*node;
+} statestack_t;
+
+static int nstatestack = 0;
+static int nastatestack;
+static statestack_t *statestack = NULL;
+
+static void push_state(node_t *node, int state)
+{
+	if(!statestack)
+	{
+		statestack = xmalloc(4 * sizeof(*statestack));
+		nastatestack = 4;
+		nstatestack = 0;
+	}
+	else if(nstatestack >= nastatestack)
+	{
+		nastatestack *= 2;
+		statestack = xrealloc(statestack, nastatestack * sizeof(*statestack));
+	}
+	statestack[nstatestack].node = node;
+	statestack[nstatestack].state = state;
+	nstatestack++;
+}
+
+static int pop_state(node_t **node, int *state)
+{
+	if(nstatestack <= 0)
+	{
+		*state = 3;
+		return 0;
+	}
+	assert(*node != NULL);
+	if(statestack[nstatestack-1].state == 8)
+		statestack[nstatestack-1].node->tcase = *node;
+	else if(statestack[nstatestack-1].state == 9)
+		statestack[nstatestack-1].node->fcase = *node;
+	*state = statestack[nstatestack-1].state;
+	*node = statestack[nstatestack-1].node;
+	return nstatestack--;
+}
+
+/* YYYY.MM.DD.hh.mm.ss */
+/* 0123456789012345678 */
+/*           111111111 */
+static char *fixup_date(const char *str)
+{
+	int i;
+	int y=1970, m=1, d=1, h=0, mi=0, s=0;
+	int l = strlen(str);
+	char date[6*16]; /* Should be wildly enough to hold 19 chars from 6 numbers with all possible errors */
+	if(l < 4 || l > 19)
+	{
+date_err:
+		stack_msg(MSG_ERR, "config: %d: Invalid date string '%s'", line_number, str);
+		return "1970.01.01.00.00.00";
+	}
+	for(i = 0; i < l; i++)
+	{
+		if(!strchr("0123456789.", str[i]))
+			goto date_err;
+	}
+	i = sscanf(str, "%d.%d.%d.%d.%d.%d", &y, &m, &d, &h, &mi, &s);
+	if(i == EOF || i < 0 || i > 6)
+		goto date_err;
+	if(i >= 1 && (y < 1970 || y > 2037))
+		goto date_err;
+	if(i >= 2 && (m < 1 || m > 12))
+		goto date_err;
+	if(i >= 3 && (d < 1 || d > 31))
+		goto date_err;
+	if(i >= 4 && (h < 0 || h > 23))
+		goto date_err;
+	if(i >= 5 && (mi < 0 || mi > 59))
+		goto date_err;
+	if(i >= 6 && (s < 0 || s > 59))
+		goto date_err;
+	sprintf(date, "%04d.%02d.%02d.%02d.%02d.%02d", y, m, d, h, mi, s);
+	return strdup(date);
+}
+
 static void config_parse(void)
 {
 	int state = 0;
 	int token;
 	int t;
 	keyword_t *kw = NULL;
+	node_t *node = NULL;
 
 	while(1)
 	{
@@ -753,17 +1010,8 @@ static void config_parse(void)
 			if(token != '=')
 			{
 				stack_msg(MSG_ERR, "config: %d: '=' expected", line_number);
-				while(1)
-				{
-					token = config_lex();
-					if(token == ';')
-					{
-						state = 0;
-						break;
-					}
-					else if(token == EOF)
-						break;
-				}
+				state = skip_to_semicolon(state);
+				break;
 			}
 			else
 				state = 2;
@@ -775,9 +1023,20 @@ static void config_parse(void)
 				state = 3;
 				break;
 			}
+			if(token == '[')
+			{
+				if(kw->type != TYPE_COLOR && kw->type != TYPE_CSTRING)
+				{
+					stack_msg(MSG_ERR, "config: %d: Conditional expression not allowed for keyword", line_number);
+					state = skip_to_semicolon(state);
+					break;
+				}
+				state = 4;
+				break;
+			}
 			if(kw->type == TYPE_FONT || kw->type == TYPE_BOOLEAN)
 				t = TYPE_NUMBER;
-			else if(kw->type == TYPE_COLOR)
+			else if(kw->type == TYPE_COLOR || kw->type == TYPE_COLORLIST || kw->type == TYPE_STRINGLIST || kw->type == TYPE_CSTRING)
 				t = TYPE_STRING;
 			else
 				t = kw->type;
@@ -795,8 +1054,11 @@ static void config_parse(void)
 				switch(kw->type)
 				{
 				case TYPE_STRING:	e = "String"; yylval.str = xstrdup("error recovery"); break;
+				case TYPE_STRINGLIST:	e = "StringL"; yylval.str = xstrdup("error recovery"); break;
+				case TYPE_CSTRING:	e = "CString"; yylval.str = xstrdup("error recovery"); break;
 				case TYPE_NUMBER:	e = "Number"; yylval.i = 0; break;
 				case TYPE_COLOR:	e = "Color"; yylval.str = xstrdup("#123456"); break;
+				case TYPE_COLORLIST:	e = "ColorL"; yylval.str = xstrdup("#123456"); break;
 				case TYPE_FONT:		e = "Font"; yylval.i = 0; break;
 				case TYPE_BOOLEAN:	e = "Boolean"; yylval.i = 0; break;
 				case TYPE_DOUBLE:	e = "Double"; yylval.d = 0.0; break;
@@ -812,6 +1074,14 @@ static void config_parse(void)
 				case TYPE_STRING:
 					*kw->confref.s = yylval.str;
 					break;
+				case TYPE_STRINGLIST:
+					kw->confref.sl->strs = xrealloc(kw->confref.sl->strs, sizeof(*kw->confref.sl->strs) * (kw->confref.sl->n + 1));
+					kw->confref.sl->strs[kw->confref.sl->n] = yylval.str;
+					kw->confref.sl->n++;
+					break;
+				case TYPE_CSTRING:
+					kw->confref.cs->str = yylval.str;
+					break;
 				case TYPE_NUMBER:
 					*kw->confref.i = yylval.i;
 					break;
@@ -824,6 +1094,11 @@ static void config_parse(void)
 				case TYPE_COLOR:
 					set_color(kw->confref.c, yylval.str);
 					break;
+				case TYPE_COLORLIST:
+					kw->confref.cl->clrs = xrealloc(kw->confref.cl->clrs, sizeof(*kw->confref.cl->clrs) * (kw->confref.cl->n + 1));
+					set_color(&kw->confref.cl->clrs[kw->confref.cl->n], yylval.str);
+					kw->confref.cl->n++;
+					break;
 				case TYPE_FONT:
 					kw->confref.f->gdfont = get_font(yylval.i);
 					break;
@@ -834,6 +1109,7 @@ static void config_parse(void)
 					yyerror("Internal error: Unknown type passed %d", kw->type);
 					break;
 			}
+			kw = NULL;
 			state = 3;
 			break;
 		case 3:
@@ -841,6 +1117,112 @@ static void config_parse(void)
 				stack_msg(MSG_ERR, "config: %d: ';' expected", line_number);
 			state = 0;
 			break;
+		case 4:
+			if(token != TYPE_STRING)
+			{
+				stack_msg(MSG_ERR, "config: %d: String expected (condition key)", line_number);
+				state = skip_to_semicolon(state);
+				break;
+			}
+			node = new_node(yylval.str);
+			state = 5;
+			break;
+		case 5:
+			if(token <= OP_FIRST || token >= OP_LAST)
+			{
+				stack_msg(MSG_ERR, "config: %d: Operator expected", line_number);
+				state = skip_to_semicolon(state);
+				break;
+			}
+			node->op = token;
+			state = 6;
+			break;
+		case 6:
+			if(token != TYPE_STRING)
+			{
+				stack_msg(MSG_ERR, "config: %d: String expected (condition)", line_number);
+				state = skip_to_semicolon(state);
+				break;
+			}
+			if(node->key == KEY_DATE)
+				node->content = fixup_date(yylval.str);
+			else
+				node->content = yylval.str;
+			state = 7;
+			break;
+		case 7:
+			if(token == '[')
+			{
+				push_state(node, 8);
+				node = NULL;
+				state = 4;
+				break;
+			}
+			if(token != TYPE_STRING)
+			{
+				stack_msg(MSG_ERR, "config: %d: String or '[' expected (true case)", line_number);
+				state = skip_to_semicolon(state);
+				break;
+			}
+			node->tcase = xmalloc(sizeof(*node->tcase));
+			if(kw->type == TYPE_COLOR || kw->type == TYPE_COLORLIST)
+			{
+				node->tcase->key = TYPE_COLOR;
+				set_color(&node->tcase->value.clr, yylval.str);
+			}
+			else
+			{
+				node->tcase->key = TYPE_STRING;
+				node->tcase->value.str = yylval.str;
+			}
+			state = 8;
+			break;
+		case 8:
+			if(token == '[')
+			{
+				push_state(node, 9);
+				node = NULL;
+				state = 4;
+				break;
+			}
+			if(token != TYPE_STRING)
+			{
+				stack_msg(MSG_ERR, "config: %d: String or '[' expected (false case)", line_number);
+				state = skip_to_semicolon(state);
+				break;
+			}
+			node->fcase = xmalloc(sizeof(*node->fcase));
+			if(kw->type == TYPE_COLOR || kw->type == TYPE_COLORLIST)
+			{
+				node->fcase->key = TYPE_COLOR;
+				set_color(&node->fcase->value.clr, yylval.str);
+			}
+			else
+			{
+				node->fcase->key = TYPE_STRING;
+				node->fcase->value.str = yylval.str;
+			}
+			state = 9;
+			break;
+		case 9:
+			if(token != ']')
+			{
+				stack_msg(MSG_ERR, "config: %d: ']' expected", line_number);
+				state = skip_to_semicolon(state);
+				break;
+			}
+			if(!pop_state(&node, &state))
+			{
+				if(kw->type == TYPE_COLOR)
+					kw->confref.c->node = node;
+				else if(kw->type == TYPE_CSTRING)
+					kw->confref.cs->node = node;
+				else
+					stack_msg(MSG_ERR, "config: %d: Color or conditional string keyword expected", line_number);
+				node = NULL;
+				kw = NULL;
+			}
+			break;
 		default:
 			yyerror("Internal error: invalid state %d", state);
 			break;
@@ -912,8 +1294,54 @@ void read_config(const char *path)
 		}
 		input_file = NULL;
 	}
+
+	if(conf.merge_from.n != conf.merge_to.n)
+	{
+		int x = conf.merge_from.n < conf.merge_to.n ? conf.merge_from.n : conf.merge_to.n;
+		stack_msg(MSG_ERR, "config: merge_from(n=%d) does not match merge_to(n=%d)", conf.merge_from.n, conf.merge_to.n);
+		conf.merge_from.n = x;
+		conf.merge_to.n = x;
+	}
+	if(conf.merge_color.n < conf.merge_from.n)
+	{
+		/* Silently extend missing merge_color statements with black */
+		int x;
+		char c[] = "#000000";
+		for(x = conf.merge_color.n; x < conf.merge_from.n; x++)
+		{
+			conf.merge_color.clrs = xrealloc(conf.merge_color.clrs, sizeof(*conf.merge_color.clrs) * (conf.merge_color.n + 1));
+			set_color(&conf.merge_color.clrs[conf.merge_color.n], c);
+			conf.merge_color.n++;
+		}
+	}
+
 #ifdef DEBUG
 	dump_config();
 #endif
 }
 
+/*
+ **************************************************************************
+ * Color reference by name for late-binding color allocation
+ **************************************************************************
+ */
+color_t *get_colorref(const char *confcolor, int idx)
+{
+	keyword_t skw;
+	keyword_t *kw;
+
+	if(!confcolor)
+		return NULL;
+
+	skw.keyword = confcolor;
+	kw = bsearch(&skw, keywords, NKEYWORDS, sizeof(keywords[0]), cmp_kw);
+	if(!kw || (kw->type != TYPE_COLOR && kw->type != TYPE_COLORLIST))
+		return NULL;
+	if(kw->type == TYPE_COLORLIST)
+	{
+		if(idx >= kw->confref.cl->n)
+			return NULL;
+		return &kw->confref.cl->clrs[idx];
+	}
+	return kw->confref.c;
+}
